@@ -29,3 +29,8 @@ RESULT_LOG = PytestDeprecationWarning(
     "--result-log is deprecated and scheduled for removal in pytest 6.0.\n"
     "See https://docs.pytest.org/en/latest/deprecations.html#result-log-result-log for more information."
 )
+
+FIXTURE_POSITIONAL_ARGUMENTS = PytestDeprecationWarning(
+    "Passing arguments to pytest.fixture() as positional arguments is deprecated - pass them "
+    "as a keyword argument instead."
+)
