@@ -8,6 +8,7 @@ from typing import Set
 import attr
 
 from ..compat import ascii_escaped
+from ..compat import ATTRS_EQ_FIELD
 from ..compat import getfslineno
 from ..compat import NOTSET
 from _pytest.outcomes import fail
@@ -292,7 +293,7 @@ class MarkGenerator:
     _config = None
     _markers = set()  # type: Set[str]
 
-    def __getattr__(self, name):
+    def __getattr__(self, name: str) -> MarkDecorator:
         if name[0] == "_":
             raise AttributeError("Marker name must NOT start with underscore")
 
@@ -367,7 +368,8 @@ class NodeKeywords(MutableMapping):
         return "<NodeKeywords for node {}>".format(self.node)
 
 
-@attr.s(cmp=False, hash=False)
+# mypy cannot find this overload, remove when on attrs>=19.2
+@attr.s(hash=False, **{ATTRS_EQ_FIELD: False})  # type: ignore
 class NodeMarkers:
     """
     internal structure for storing marks belonging to a node
