@@ -1,5 +1,6 @@
 from pprint import pprint
 from typing import Optional
+from typing import Union
 
 import py
 
@@ -267,7 +268,8 @@ class TestReport(BaseReport):
             if not isinstance(excinfo, ExceptionInfo):
                 outcome = "failed"
                 longrepr = excinfo
-            elif excinfo.errisinstance(skip.Exception):
+            # Type ignored -- see comment where skip.Exception is defined.
+            elif excinfo.errisinstance(skip.Exception):  # type: ignore
                 outcome = "skipped"
                 r = excinfo._getreprcrash()
                 longrepr = (str(r.path), r.lineno, r.message)
@@ -431,7 +433,7 @@ def _report_kwargs_from_json(reportdict):
                 reprlocals=reprlocals,
                 filelocrepr=reprfileloc,
                 style=data["style"],
-            )
+            )  # type: Union[ReprEntry, ReprEntryNative]
         elif entry_type == "ReprEntryNative":
             reprentry = ReprEntryNative(data["lines"])
         else:
